<?php 

namespace Teknisa\MAN\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services;

class CadCampanhaBeneficio extends \Teknisa\MAN\Controllers\TeknisaCrud {
	// -----   DATASOURCE NAMES   -----
	const WIDGETPRINCIPAL = 'cadCampanhaBeneficio';
	const FILIALPARTICIPANTE = 'cadCampanhaBeneficio_filialParticipante';
	const PRODUTOPARTICIPANTE = 'cadCampanhaBeneficio_produtoParticipante';
	const BENEFICIOPRODUTO = 'cadCampanhaBeneficio_beneficioProduto';
	const PRODUTOCONTEMPLADO = 'cadCampanhaBeneficio_produtoContemplados';
	const GETFILIAL = 'getFilialParticipante';
    
    // -----   PROTECTED VARIABLES   -----
    protected $dataSourceName = '';
    protected $paramBagColumns = [];
	protected $entityManager; 
	protected $util;
	protected $fidelidade;

	public function __construct($crudService,Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, Utilities\Util $util, Services\Fidelidade $fidelidade)  {
	    $this->dataSourceManager = $dataSourceManager;
	    $this->parameterBag = $parameterBag;
	    $this->crudService =$crudService;
		$this->entityManager = $entityManager;
		$this->util = $util;    
		$this->fidelidade = $fidelidade;
	}
	
	// -----   GET/FIND FUNCTIONS   -----
	public function getCampanhas (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::WIDGETPRINCIPAL;
    	$this->parameterBagColumns = array('NRORG');
        parent::find($request, $response);
	}
	
	public function getFiliais (DTO\Request\Filter $request, DTO\Response $response) {
		$this->dataSourceName = $this::FILIALPARTICIPANTE;
    	$this->parameterBagColumns = array('CDCAMPANHA', 'NRORG');
    	parent::find($request, $response);
	}
	
	public function getFilial (DTO\Request\Filter $request, DTO\Response $response) {
		$this->dataSourceName = $this::GETFILIAL;
    	$this->parameterBagColumns = array('CDCAMPANHA', 'NRORG', 'CDOPERADOR');
    	parent::find($request, $response);
	}
	
	public function getProdutosParticipantes (DTO\Request\Filter $request, DTO\Response $response) {
		$this->dataSourceName = $this::PRODUTOPARTICIPANTE;
    	$this->parameterBagColumns = array('NRORG', 'CDPRODUTO', 'CDCAMPANHA', 'NRSEQPRODCAMP', 'DTINIVIGPROD', 'DTFIMVIGPROD', 'DTINCLUSAO', 'DTULTATU', 'NRORGINCLUSAO', 'CDOPERINCLUSAO', 'NRORGULTATU', 'CDOPERULTATU', 'NRDIASVALBENECAMP', 'QTBENEFICIOCAMP');
    	parent::find($request, $response);
	}
	
	public function getBeneficiosProduto (DTO\Request\Filter $request, DTO\Response $response) {
		$this->dataSourceName = $this::BENEFICIOPRODUTO;
		$this->parameterBagColumns = array('NRORG', 'NRSEQPRODCAMP', 'CDCAMPANHA');
		parent::find($request, $response);
	}
	
	public function getProdutosContemplados (DTO\Request\Filter $request, DTO\Response $response) {
		$this->dataSourceName = $this::PRODUTOCONTEMPLADO;
		$this->parameterBagColumns = array('NRORG', 'NRSEQBENEFICIO', 'CDCAMPANHA', 'NRSEQPRODCAMP', 'CDPRODUTO');
		parent::find($request, $response);
	}
	
	// -----   SAVE FUNCTIONS   -----
	
	public function saveCampanha (DTO\Request\DataSet $request, DTO\Response $response) {
		$this->dataSourceName = $this::WIDGETPRINCIPAL;
		$conditions = $this->util->getParamsAsArray($request)[0];
		if($conditions['__is_new'] == true) {	
			$params = array(
			    ':CDCAMPANHA' => $conditions['CDCAMPANHA'],
			    ':NRORG' => $conditions['NRORG']);
			$count = $this->entityManager->getConnection()->fetchAll(self::COUNT_CDCAMPANHA, $params);
			if($count[0]['COUNT(*)'] > 0){
			    throw new \Exception("Código já existente!");   
			    $error[0] = "Código já existente";
			    $response->addDataSet(new DataSet('Error', $error));
			}
			else{
        		$this->saveWithoutNotification($request, $response);
			}
		}else{
		    $this->saveWithoutNotification($request, $response);
		}
	}
	
	public function saveFilial(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this::FILIALPARTICIPANTE;
    	
        try{
            $conditions = $request->getDataSet();
            $aux =  $conditions->getRows()[0]['CDFILIAL'];
            for($i=0;$i<count($aux);$i++){
                $conditions ->getRows()[0]['CDFILIAL'] = $aux[$i];
                $conditions->setDataSourceName($this->dataSourceName);
                $persistedRows = $this->dataSourceManager->persist($conditions);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveProdutoParticipante(DTO\Request\DataSet $request, DTO\Response $response) {
    	$this->dataSourceName = $this::PRODUTOPARTICIPANTE;
        $rows = $request->getDataSet()->getRows()[0];  
        
        if($rows['__is_new'] == true) {
            
            if($rows['CDARVPRODINI'] == $rows['CDARVPRODFIN']) {
            	$produtos[0]['CDPRODUTO'] = $rows['CDARVPRODINI'];
            } else {
	            $produtos = $this->fidelidade->getProductsBetweenTwo($rows);
            }
            
            $params = array("CDCAMPANHA" => $rows['CDCAMPANHA']);
            $maxSeqCodeProduct = floatval($this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_MAX_SEQCODE_PRODUCT, $params)[0]['MAX']);
            $maxSeqCodeProduct++;
            
            if($maxSeqCodeProduct == null) {
                $maxSeqCodeProduct = 1;
            }
            
           	for($i = 0; $i < count($produtos); $i++, $maxSeqCodeProduct++) {
           		
           		$this->fidelidade->incluirProduto(
           			$rows['CDCAMPANHA'],
           			str_pad($maxSeqCodeProduct, 10, '0', STR_PAD_LEFT),
           			$produtos[$i]['CDPRODUTO'],
           			'P',
           			$rows['VRPONTOPROD'],
           			$rows['DTINIVIGPROD'],
					$rows['DTFIMVIGPROD'],
					$rows['NRORG'],
					$rows['NRORGINCLUSAO'],
					$rows['CDOPERINCLUSAO'],
					$rows['NRORGULTATU'],
					$rows['CDOPERULTATU']
           		);
           	}
            	
        } else {
            $this->saveWithoutNotification($request, $response);
        }
    }
    
    public function saveBeneficioProduto (DTO\Request\DataSet $request, DTO\Response $response) {
    	$this->dataSourceName = $this::BENEFICIOPRODUTO;
    	$this->saveWithoutNotification($request, $response);
    }
    
    public function saveProdutoContemplado($request, $response) {
		$this->dataSourceName = $this::PRODUTOCONTEMPLADO;
		$conditions = $request->getDataSet();
        $rows = $conditions->getRows()[0];
        
        $CDARVPROD = $rows['CDARVPROD'];
        $NRSEQPRODCAMP = $rows['NRSEQPRODCAMP'];
        $CDCAMPANHA = $rows['CDCAMPANHA'];
        $NRSEQBENEFICIO = $rows['NRSEQBENEFICIO'];
    	if ( $rows['NMTPSELECT'] == 'I'){
    		$produtos = $this->fidelidade->getProdutosContemplados($rows['CDARVPRODINI'], $rows['CDARVPRODFIN'], $rows['NRORG']);
    	}
    	else{
    		$produtos = [];
    		foreach ($rows['PRODUTOS'] as $key => $PRODUTO) {
    			$array = array(
    				'CDPRODUTO' => $PRODUTO
    			);
    			array_push($produtos , $array);
    		}
    	}
    	unset($rows['PRODUTOS']);
        unset($rows['CDARVPROD']);
        unset($rows['NRSEQPRODCAMP']);
        unset($rows['CDCAMPANHA']);
        unset($rows['NRSEQBENEFICIO']);
        
        $produtos = array_values($produtos);
        asort($produtos);
        $arrayAux = [];
        $rowsToSave = [];
        
        for($i=0; $i<count($produtos); $i++) {
            array_push($rowsToSave, clone $rows);
            $rowsToSave[$i]['NRSEQPRODCAMP'] = $NRSEQPRODCAMP;
            $rowsToSave[$i]['CDCAMPANHA'] = $CDCAMPANHA;
            $rowsToSave[$i]['NRSEQBENEFICIO'] = $NRSEQBENEFICIO;
            $rowsToSave[$i]['CDPRODUTO'] = $produtos[$i]['CDPRODUTO'];
	        $newRequest = new DTO\Request\DataSet (
	            new DataSet($this->dataSourceName, $rowsToSave) , 
	            $request->getMethod(),
	            $request->getRoutePath(),
	            $request->getUserId()
	        );
        	$this->saveWithoutNotification($newRequest, $response);
        	unset($rowsToSave[$i]);
        }
	}
	
	// -----   DELETE FUNCTIONS   -----
	
	public function deleteCampanha (DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = $this::WIDGETPRINCIPAL;
        parent::delete($request,$response);
	}
	
	public function deleteFilial (DTO\Request\DataSet $request, DTO\Response $response) {
		$this->dataSourceName = $this::FILIALPARTICIPANTE;
		$this->deleteWithoutNotification($request, $response);
	}
	
	public function deleteProdutoParticipante (DTO\Request\DataSet $request, DTO\Response $response) {
		$this->dataSourceName = $this::PRODUTOPARTICIPANTE;
		$this->deleteWithoutNotification($request, $response);
	}
	
	public function deleteBeneficioProduto (DTO\Request\DataSet $request, DTO\Response $response) {
		$this->dataSourceName = $this::BENEFICIOPRODUTO;
		$this->deleteWithoutNotification($request, $response);
	}
	
	public function deleteProdutoContemplado (DTO\Request\DataSet $request, DTO\Response $response) {
		$this->dataSourceName = $this::PRODUTOCONTEMPLADO;
		$this->deleteWithoutNotification($request, $response);
	}
	
	// -----   SPECIAL FUNCTIONS -----
	
	private function getNewSeqCode ($qtdString, $nrorg) {
		$params = array("NRORG" => $nrorg);
		$seqcode = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_MAX_CODE_CAMPANHA, $params)[0]['MAX'];
		$seqcode++;
		$seqcode = str_pad($seqcode, $qtdString, "0", STR_PAD_LEFT);
		return $seqcode;
	}
	
	private function saveWithoutNotification($request, $response) {
		try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	private function deleteWithoutNotification ($request, $response) {
		try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	// ------------ GENERATE CÓDIGO -------------
	
	public function criaCodigoCampanha(DTO\Request $request, DTO\Response $response){
	    $params = $this->util->getParamsAsArray($request);
	    $seqcode[0] = $this->getNewSeqCode(10, $params['NRORG']);
	    $response->addDataSet(new DataSet('CodigoCampanha', $seqcode));
	}
	
	 public function addParams($conditions,$param){
        
        if(!array_key_exists($param, $conditions)) {
            $conditions[$param] = '';
            $conditions['T'.$param] = 'T';
        }else{
            $conditions[$param] = $conditions[$param];
            $conditions['T'.$param] = '';
        }
        return $conditions;
    }
	
	const COUNT_CDCAMPANHA = "SELECT COUNT(*)
                                FROM CAMPANHA 
                              WHERE NRORG = :NRORG
                              AND CDCAMPANHA = :CDCAMPANHA";
}